#--
# Copyright (c) 2010 Moodstocks SAS
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# or in the LICENSE file which you should have received as part of
# this distribution.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#++

module MNotes class User
  
  PW = {}
  
  class << self
    
    def create(uid, secret, email='test@example.com')
      u = new(uid, secret)
      r = Client::post( "#{API}/user", :query => { :uid => uid, :secret => secret, :email => email }, :body => '' )
      raise r.inspect unless r.code.to_i == 200
      PW[uid] = secret
      @lastuser = u unless @lastuser
      u
    end
    
    def last
      @lastuser
    end
    
    def retrieve(uid, secret=nil)
      secret = PW[uid] unless secret
      u = new(uid, secret)
      u.jbody(:get, "#{u.rs}/echo")
      PW[uid] = secret
      @lastuser = u unless @lastuser
      u
    end
    
    def set_last(u)
      @lastuser = u
    end
    
  end # class << self
  
  attr_reader :uid, :secret, :rs
  
  def delete
    jbody(:delete, "#{@rs}")
    true
  end
    
  def initialize(uid, secret)
    @uid, @secret = uid, secret
    @rs = "user/#{uid}"
  end
  
  def jbody(r, path=nil, params={})
    Client::jbody(r, path, @uid, @secret, params)
  end
  
  def notes(group=false)
    q = group ? {:group => group} : {}
    jbody(:get, "#{@rs}/note", :query => q)[:notes].map_mr(:new, Note)
  end
  
  def query(fpath)
    r = Client::query(fpath, @uid, @secret)
    SearchResult.new(r, self)
  end
  
  def unvote(obj)
    obj = obj.id if obj.is_a?(MNotes::Object)
    jbody(:delete, "object/#{obj}/vote")
    true
  end
  
  def update_note(id, text, location=nil)
    params = {:body => text}
    params.merge!(:query => location) if location
    jbody(:put, "note/#{id}", params)
    true
  end
  
  def vote(obj, vote)
    raise unless %w{up down}.include?(vote = vote.to_s)
    obj = obj.id if obj.is_a?(MNotes::Object)
    jbody(:put, "object/#{obj}/vote/#{vote}")
    true
  end
  
end end
